/*
 * 2003 (c) All rights reserved.
 */
package portaview;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.net.InetAddress;
import java.util.List;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTextArea;
import javax.swing.SwingConstants;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.PosixParser;

import portaview.model.AlbumModel;
import portaview.model.PortaViewCollection;
import portaview.model.PortaViewModel;
import portaview.server.PortaViewClient;
import portaview.server.PortaViewServer;
import portaview.server.Repository;
import portaview.view.AlbumPanel;
import portaview.view.PortaViewPanel;

/**
 * This is the main PortaView class.  Essentially it contains two things.  
 * The PortaView Manager and PortaView displayer.  It also initializes the
 * necessary XML-RPC server so remote host can talk to this instance.
 *
 * @author William Lee
 *
 */
public class PortaViewMain extends DefaultFrame
{
    private static int WIDTH = 1024;
    private static int HEIGHT = 700;

    private static JTextArea _debugPanel = new JTextArea(70, 1000);
    private Displayer displayer = null;
    private Previewer preview = null;
    private PortaViewModel thisPortaViewModel = null;
    private PortaViewPanel portaViewPanel = null;
    private AlbumPanel albumPanel = null;
    private List albums = null;
    /**
     * Indicates whether the display is active.
     */
    private boolean displayActive = false;

    /**
     * Constructs the main PortaView application, given the command-line arguments.
     * @param args command-line arguments.
     * @throws Exception if the initialization routine goes wrong.
     */
    public PortaViewMain(String[] args) throws Exception
    {
        super();
        initRegistry(args);
        albums = new Vector();
        thisPortaViewModel =
            new PortaViewModel(
                Registry.getServer().getName(),
                Registry.getServer().getInetAddress(),
                AlbumModel.getEmptyAlbumModel());
        initDisplayer();
        setTitle(
            "PortaView Manager: Current PortaView: ("
                + Registry.getServer().getName()
                + ")");
        initManager();
    }

    /**
     * Initializes the displayer.
     *
     */
    public void initDisplayer()
    {
        displayer = new Displayer(getThisPortaViewModel());
    }

    /**
     * Initializes the manager.
     */
    private void initManager()
    {
        setSize(WIDTH, HEIGHT);

        // Generate the PortaViewPanel
        JPanel pvpanel = new JPanel(new BorderLayout());
        pvpanel.add(
            WidgetFactory.createCenteredLabel("PortaViews"),
            BorderLayout.NORTH);
            /*
        pvpanel.add(
            new JLabel("( * ) You are at this PortaView"),
            BorderLayout.SOUTH);
            */
        portaViewPanel =
            new PortaViewPanel(Registry.getServer().getRegisteredPortaViews());
        portaViewPanel.setPreferredSize(new Dimension(150, 500));
        pvpanel.add(portaViewPanel, BorderLayout.CENTER);

        // Generate the AlbumPanel
        JPanel apanel = new JPanel();
        apanel.setLayout(new BorderLayout());
        JLabel albl = new JLabel("Albums");
        albl.setHorizontalAlignment(SwingConstants.CENTER);
        albl.setVerticalAlignment(SwingConstants.CENTER);
        albl.setBorder(BorderFactory.createEtchedBorder());
        apanel.add(albl, BorderLayout.NORTH);
        albumPanel = new AlbumPanel(Registry.getServer().getAlbums());
        //albumPanel.setPreferredSize(new Dimension(150, 500));
        apanel.add(albumPanel, BorderLayout.CENTER);

        // this is experimental implementation of the display panel.
        PortaViewModel pvm = getThisPortaViewModel();
        // Initialize the displayer controller
        preview = new Previewer(pvm);

        JSplitPane rightPane =
            new JSplitPane(
                JSplitPane.VERTICAL_SPLIT,
                preview.getView(),
                apanel);

        JSplitPane mainPane =
            new JSplitPane(JSplitPane.HORIZONTAL_SPLIT, pvpanel, rightPane);
        getContentPane().add(mainPane);
    }

    /**
     * Prints out a debug message.
     * @param str message to print
     */
    public static void debug(String str)
    {
        System.out.println(str);
        _debugPanel.setText(_debugPanel.getText() + str + "\n");
    }

    /**
     * Constructs a debug panel.
     */
    public static void configureDebugPanel()
    {
        _debugPanel.setPreferredSize(new Dimension(300, 50));
    }

    /**
     * Sets up the PortaView server based on the command line argument
     * @param cmd the command line arguments.
     */
    private void setupServer(CommandLine cmd) throws Exception
    {
        boolean isMaster = cmd.hasOption("m");
        String name = "(No Name)";
        String masterIP = "127.0.0.1";
        String pvIP = "127.0.0.1";
        String repo = "albums";
        int port = 8888;

        if (cmd.hasOption("n"))
        {
            name = cmd.getOptionValue("n");
        }
        if (cmd.hasOption("c"))
        {
            masterIP = cmd.getOptionValue("c");
        }
        if (cmd.hasOption("i"))
        {
            pvIP = cmd.getOptionValue("i");
        }
        if (cmd.hasOption("a"))
        {
            repo = cmd.getOptionValue("a");
        }
        // Creates the Server
        Registry.setServer(
            new PortaViewServer(
                name,
                InetAddress.getByName(pvIP),
                InetAddress.getByName(masterIP),
                new Repository(new File(repo)),
                Consts.XML_RPC_PORT));
        Registry.setClient(new PortaViewClient());
    }

    /**
     * Parses the command line argument.
     * @param args arguments passed from the command line arguments.
     */
    private void initRegistry(String[] args) throws Exception
    {
        CommandLineParser parser = new PosixParser();
        Options options = new Options();
        CommandLine cmd = null;
        Registry.reset();
        // parses the command line
        options.addOption("m", "master", false, "set this as master device.");
        options.addOption("n", "name", true, "name for the PortaView.");
        options.addOption("i", "ip", true, "this PortaView's address.");
        options.addOption(
            "c",
            "connect",
            true,
            "connect to address (master ip address).");
        options.addOption(
            "a",
            "album-dir",
            true,
            "the album directory (only if this PortaView is a master).");
        try
        {
            cmd = parser.parse(options, args);
            setupServer(cmd);
            // get whether this is a master
            Registry.setMaster(cmd.hasOption("m"));
            // set the main instance to the registry.
            Registry.setMainApplication(this);
        }
        catch (ParseException exp)
        {
            System.err.println("Illegal argument. Reason: " + exp.getMessage());
            throw exp;
        }
        catch (Exception e)
        {
            System.err.println(
                "May have illegal address or unknown problem. Reason: "
                    + e.toString());
            throw e;
        }
    }

    /**
     * Starting point for PortaView's main execution.
     * @param args command-line arguments
     */
    public static void main(String[] args)
    {
        String[] gargs = getGUIArgs(args);
        PortaViewMain pv = null;
        try
        {
            pv = new PortaViewMain(gargs);
            pv.start();
            pv.setVisible(true);
        }
        catch (Exception e)
        {
            if (pv != null)
                pv.shutdown();
            System.err.println(e.toString());
            e.printStackTrace();
            System.exit(1);
        }
    }

    /**
     * If the user does not type in a command line argument, pop up a box 
     * for them to type it in.
     * @return the new command line argument after prompting the user
     * if args does not contain anything.
     */
    public static String[] getGUIArgs(String[] args)
    {
        if (args.length == 0)
        {
            // Ask whether this is a master PortaView
            boolean isMaster = true;
            int answer =
                JOptionPane.showConfirmDialog(
                    null,
                    "Is this a Master PortaView?\n"
                        + "(If this PortaView is a Slave PortaView, select 'No'",
                    "Master PortaView?",
                    JOptionPane.YES_NO_OPTION);
            if (answer == JOptionPane.NO_OPTION)
            {
                isMaster = false;
            }
            String name = "Untitled";
            name = JOptionPane.showInputDialog("Name of this PortaView?", name);
            String ip = "127.0.0.1";
            ip =
                JOptionPane.showInputDialog(
                    "PortaView Internet Protocol Address:\n"
                        + "This address should be an IP address that is\n"
                        + "visible to external network.",
                    ip);
            String masterIp = ip;
            if (!isMaster)
            {
                masterIp =
                    JOptionPane.showInputDialog(
                        "Master PortaView IP Address",
                        "127.0.0.1");
            }

            String repo = "src/test/test/portaview/images";
            if (isMaster)
                repo = JOptionPane.showInputDialog("Album directory?", repo);
            if (isMaster)
            {
                String[] rtnMaster =
                    { "-m", "-n", name, "-i", ip, "-c", masterIp, "-a", repo };
                return rtnMaster;
            }
            else
            {

                String[] rtnSlave = { "-n", name, "-i", ip, "-c", masterIp };
                return rtnSlave;
            }

        }
        return args;
    }

    /**
     * Populate the frame with the main panel.
     * @param panel
     */
    private static void populateFrame(JFrame frame, Component c)
    {
        frame.getContentPane().setLayout(new BorderLayout());
        frame.getContentPane().add(c, BorderLayout.CENTER);
        JScrollPane scroll = new JScrollPane(_debugPanel);
        scroll.setPreferredSize(new Dimension(300, 100));
        scroll.setHorizontalScrollBarPolicy(
            JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
        frame.getContentPane().add(scroll, BorderLayout.SOUTH);
    }

    /**
     * Starts the main application.
     */
    public void start() throws Exception
    {
        Registry.getServer().start();
        PortaViewCollection pvc =
            Registry.getServer().getRegisteredPortaViews();
        pvc.add(thisPortaViewModel);
        // force the views that based on this portview model to refresh
        thisPortaViewModel.forceNotifyObservers();
        //select the first PortaView 
        portaViewPanel.selectFirst();
        if (!Registry.isMaster())
        {
            Registry.getClient().registerWithMaster();
        }

        // Initailize the preview controller
        PortaViewModel pvm = getThisPortaViewModel();
        // populate the album panel.  It'll take a long time.
        Registry.getServer().setAlbums(
            Registry.getClient().getAlbumsFromMaster());
        albumPanel.refresh();
    }

    /**
     * Closes all the resources that this PortaView opens.
     */
    public void shutdown()
    {
        // If this is a slave server, then we unregister with the master
        if (!Registry.isMaster())
            Registry.getClient().unregisterWithMaster();
        Registry.getServer().shutdown();
    }

    /**
     * The listener that handles the windows exit event.  It'll try to shutdown the 
     * server and exit.
     *
     * @return a window listener that just call System.exist(0);
     */
    protected WindowAdapter getDefaultWindowAdapter()
    {
        WindowAdapter rtn = (new WindowAdapter()
        {
            public void windowClosing(WindowEvent e)
            {
                Log.debug("Shutting down the server.");
                shutdown();
                System.exit(0);
            }

            public void windowDeiconified(WindowEvent e)
            {
            }

            public void windowIconified(WindowEvent e)
            {
            }
        });
        return rtn;
    }

    /**
     * @return
     */
    public Displayer getDisplayer()
    {
        return displayer;
    }

    /**
     * @return
     */
    public Previewer getPreview()
    {
        return preview;
    }

    /**
     * @param displayer
     */
    public void setDisplayer(Displayer displayer)
    {
        this.displayer = displayer;
    }

    /**
     * @param previewer
     */
    public void setPreview(Previewer previewer)
    {
        preview = previewer;
    }

    /**
     * @return
     */
    public PortaViewModel getThisPortaViewModel()
    {
        return thisPortaViewModel;
    }

    /**
     * @param model
     */
    public void setThisPortaViewModel(PortaViewModel model)
    {
        thisPortaViewModel = model;
    }

    /**
     * @return
     */
    public boolean isDisplayActive()
    {
        return displayActive;
    }

    /**
     * @param b
     */
    public void setDisplayActive(boolean b)
    {
        displayActive = b;
        // if deactive the display, we need to refresh the preview panel and
        // all the portaviews icons.
        if (!b)
        {
            getPreview().getView().refresh();
            portaViewPanel.refresh();
        }
    }

}
