/*
 * Created on Oct 23, 2003
 */
package portaview.model;

import java.net.InetAddress;
import java.util.Observable;
import java.util.Observer;

import portaview.Registry;
import portaview.RemoteUpdateController;

/**
 * Represents a PortaView.  This main contains two parts.  One is the SlideSettings, 
 * which controls the settings.  The other is the image associated with the PortaView.
 * @author William Lee
 */
public class PortaViewModel extends NameModel implements Observer
{
    public static final String IMAGE_CHANGED = "image_changed";
    public static final String SETTINGS_CHANGED = "settings_changed";

    private InetAddress inetAddress = null;
    private AlbumModel album = null;
    private SlideSettings slideSettings = null;
    private PortaViewImageModel imageModel = null;
    private RemoteUpdateController remoteController = null;

    public PortaViewModel(String name, InetAddress address, AlbumModel album)
    {
        super(name);
        this.inetAddress = address;
        this.album = album;
        this.slideSettings = new SlideSettings();
        this.imageModel = new PortaViewImageModel(this);
        // attach a controller so we can update the remove PortaView.
        remoteController = new RemoteUpdateController(this);
    }

    /**
     * Moves to the next slide.
     *
     */
    public void nextSlide()
    {
        slideSettings.setIndex(
            (slideSettings.getIndex() + 1) % slideSettings.getTotal());
        setSlideSettings(slideSettings);
    }
    
    /**
     * Moves to the previous slide.
     *
     */
    public void prevSlide()
    {
        if (slideSettings.getIndex() == 0)
        {
            slideSettings.setIndex(slideSettings.getTotal() - 1);
        }
        else
        {
            slideSettings.setIndex((slideSettings.getIndex() - 1) % slideSettings.getTotal());
        }
        // we need to do this in order to notify the change.
        setSlideSettings(slideSettings);
    }
    
    /**
     * Sets the PortaView pause play.
     *
     */
    public void play()
    {
        slideSettings.setPlayState(SlideSettings.PLAY);
        setSlideSettings(slideSettings);
    }

    /**
     * Sets the PortaView to pause.
     */
    public void pause()
    {
        slideSettings.setPlayState(SlideSettings.PAUSE);
        setSlideSettings(slideSettings);
    }

    /**
     * @return the internet address for a given PortaView
     */
    public InetAddress getInetAddress()
    {
        return inetAddress;
    }

    /**
     * Sets the Ip for the PortaView.
     * @param address the internet address for the PortaView
     */
    public void setInetAddress(InetAddress address)
    {
        inetAddress = address;
    }

    /**
     * @return the AlbumModel that thie PortaView is showing.
     */
    public AlbumModel getAlbum()
    {
        return album;
    }

    /**
     * @param model the album model.
     */
    public void setAlbum(AlbumModel model)
    {
        album = model;
    }
    /**
     * @return the slide settings, which contain most internal state for a PortaView.
     */
    public SlideSettings getSlideSettings()
    {
        return slideSettings;
    }

    /**
     * Sets the slide settings and notify all the observers about the change.
     * @param slideSettings slide settings to set.
     */
    public void setSlideSettings(SlideSettings slideSettings)
    {
        this.slideSettings = slideSettings;
        // Notify the observers for changes.
        setChanged();
        notifyObservers(SETTINGS_CHANGED);
    }

    /**
     * @return the image model
     */
    public PortaViewImageModel getImageModel()
    {
        return imageModel;
    }

    /**
     * Sets the image model.
     * @param imageModel model to set.
     */
    public void setImageModel(PortaViewImageModel imageModel)
    {
        this.imageModel = imageModel;
    }

    /**
     * Underlying models have changed.  Notify the observers to this displayer model.
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable arg0, Object arg1)
    {
        setChanged();
        notifyObservers(arg1);
    }

    /**
     * Update this portaview model with the remote portaview's state.
     *
     */
    public void updateRemote()
    {
        Registry.getClient().updatePortaViewModel(this);
    }
}
