/*
 * Created on Sep 29, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.model;

import java.awt.Color;
import java.util.Hashtable;

/**
 * Settings for the slideshow.  This contains most of the state in the PortaView.  This
 * object is transfered over to remote PortaView in order to change their settings.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: SlideSettings.java,v 1.6 2003/12/10 06:18:57 wlee Exp $
 */
public class SlideSettings
{
    public static final String SLIDE = "slide"; // slide name
    public static final String INDEX = "index"; // index in side
    public static final String TOTAL = "total"; // total number of slides
    public static final String SPEED = "speed";
    public static final String TRANSPARENCY = "transparency";
    public static final String PLAY_STATE = "state";
    // speed 0 = stop, >0 = second for the next slide.
    public static final String BORDER_RED = "borderRed"; // border red
    public static final String BORDER_GREEN = "borderGreen"; // border green
    public static final String BORDER_BLUE = "borderBlue"; // border blue
    public static final String BORDER = "border"; // border thickness
    public static final int PLAY = 0; // constant for play state
    public static final int PAUSE = 1; // constant for pause state

    private String slide = "";
    private int index = 0;
    private int total = 0;
    private int speed = 15;
    private Color borderColor = Color.BLACK;
    private int border = 0;
    private int playState = PAUSE;
    private int transparency = 10;

    public SlideSettings()
    {
    }

    /**
     * Returns true if the given slide settings is exactly the same.  This can be used
     * for checking whether the remote PortaView needs to be updated.
     * @param ss slide settings to test.
     * @return true if ss equals the current object.
     */
    public boolean equals(SlideSettings ss)
    {
        if (!ss.getSlide().equals(slide))
            return false;
        if (ss.getIndex() != index)
            return false;
        if (ss.getTotal() != total)
            return false;
        if (ss.getSpeed() != speed)
            return false;
        if (!ss.getBorderColor().equals(borderColor))
            return false;
        if (ss.getBorder() != border)
            return false;
        if (ss.getPlayState() != playState)
            return false;
        return true;
    }

    public boolean equals(Object ss)
    {
        return equals((SlideSettings) ss);
    }

    /**
     * Serializes to string for pretty print out.
     */
    public String toString()
    {
        StringBuffer sb = new StringBuffer();
        sb.append(slide);
        sb.append(":");
        sb.append(index);
        sb.append("/");
        sb.append(total);
        sb.append(":");
        sb.append("speed=");
        sb.append(speed);
        sb.append(":");
        sb.append("color=");
        if (borderColor != null)
        {
            sb.append(":");
            sb.append(borderColor.toString());
        }
        sb.append(":");
        sb.append("border=");
        sb.append(border);
        sb.append(":");
        sb.append("trans=");
        sb.append(transparency);
        sb.append(":");
        sb.append("playState=");
        if (playState == PLAY)
            sb.append("play");
        else
            sb.append("pause");
        return sb.toString();
    }
    /** 
     * Given a Hashtable that represents this object (got form XML-RPC mostly), convert
     * and set this object.
     */
    public SlideSettings(Hashtable def)
    {
        this.slide = (String) def.get(SLIDE);
        Integer val = ((Integer) def.get(INDEX));
        if (val != null)
            this.index = val.intValue();
        val = ((Integer) def.get(TOTAL));
        if (val != null)
            this.total = val.intValue();
        val = ((Integer) def.get(SPEED));
        if (val != null)
            this.speed = val.intValue();

        Integer r, g, b;
        r = ((Integer) def.get(BORDER_RED));
        g = ((Integer) def.get(BORDER_GREEN));
        b = ((Integer) def.get(BORDER_BLUE));
        if (r != null && g != null && b != null)
        {
            this.borderColor =
                new Color(r.intValue(), g.intValue(), b.intValue());
        }

        val = ((Integer) def.get(BORDER));
        if (val != null)
            this.border = val.intValue();
        val = ((Integer) def.get(PLAY_STATE));
        if (val != null)
            this.playState = val.intValue();
        val = ((Integer) def.get(TRANSPARENCY));
        if (val != null)
            this.transparency = val.intValue();
    }

    /**
     * @return a Hashtable for XML-RPC calls.
     */
    public Object toHashtable()
    {
        Hashtable rtn = new Hashtable();
        rtn.put(SLIDE, slide);
        rtn.put(INDEX, new Integer(index));
        rtn.put(TOTAL, new Integer(total));
        rtn.put(SPEED, new Integer(speed));
        rtn.put(BORDER_RED, new Integer(borderColor.getRed()));
        rtn.put(BORDER_GREEN, new Integer(borderColor.getGreen()));
        rtn.put(BORDER_BLUE, new Integer(borderColor.getBlue()));
        rtn.put(BORDER, new Integer(border));
        rtn.put(PLAY_STATE, new Integer(playState));
        rtn.put(TRANSPARENCY, new Integer(transparency));
        return rtn;
    }

    /**
     * @return the border width.
     */
    public int getBorder()
    {
        return border;
    }

    /**
     * Sets border width.
     * @param border the border width.
     */
    public void setBorder(int border)
    {
        this.border = border;
    }

    /**
     * @return the border color
     */
    public Color getBorderColor()
    {
        return borderColor;
    }

    /**
     * Sets border color
     * @param borderColor color to set.
     */
    public void setBorderColor(Color borderColor)
    {
        this.borderColor = borderColor;
    }

    /**
     * @return the index of the picture shown in the current slide.
     */
    public int getIndex()
    {
        return index;
    }

    /**
     * Sets the picture index.
     * @param index index of the picture.
     */
    public void setIndex(int index)
    {
        this.index = index;
    }

    /**
     * @return the slide's name.
     */
    public String getSlide()
    {
        return slide;
    }

    /**
     * Sets the slide's name.
     * @param slide slide's name.
     */
    public void setSlide(String slide)
    {
        this.slide = slide;
    }

    /**
     * @return the total number of slides it is showing.
     */
    public int getTotal()
    {
        return total;
    }

    /**
     * Sets the total number of slides.
     * @param total total number of slides.
     */
    public void setTotal(int total)
    {
        this.total = total;
    }

    /**
     * @return the play speed.
     */
    public int getSpeed()
    {
        return speed;
    }

    /**
     * Sets the play speed.
     * @param speed play speed.
     */
    public void setSpeed(int speed)
    {
        this.speed = speed;
    }

    /**
     * @return the play state.  This can be PLAY or PAUSE.
     */
    public int getPlayState()
    {
        return playState;
    }

    /**
     * Sets the play state.
     * @param i play state, this can be PLAY or PAUSE.
     */
    public void setPlayState(int i)
    {
        playState = i;
    }

    /**
     * @return the alpha transparency.  This number goes from 0 to 10.
     */
    public int getTransparency()
    {
        return transparency;
    }

    /**
     * Sets the alpha transparency
     * @param i alpha transparency.  This number goes from 0 to 10.
     */
    public void setTransparency(int i)
    {
        transparency = i;
    }

}
