/*
 * Created on Oct 6, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.server;

import java.util.Hashtable;
import java.util.List;
import java.util.Vector;

import org.apache.xmlrpc.AsyncCallback;
import org.apache.xmlrpc.XmlRpcClient;

import portaview.Consts;
import portaview.Log;
import portaview.Registry;
import portaview.model.Album;
import portaview.model.AlbumCollection;
import portaview.model.AlbumModel;
import portaview.model.PhotoModel;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;
import portaview.util.ImageUtils;

/**
 * Provides the API for client calls to remote PortaViews.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: PortaViewClient.java,v 1.10 2003/12/10 06:18:57 wlee Exp $
 */
public class PortaViewClient
{
    /**
     * The viewer server that contains all data about the connections.  Note that the client
     * depends on the server instance recorded in the registry.  You need to call the 
     * Registry.setSever() first
     */
    public PortaViewClient()
    {
    }

    /**
     * Updates the remote PortaView model.  We usually do this when the model has changed
     * and we want to notify the remote PortaView about this change.
     * @param model the model to update.
     */
    public void updatePortaViewModel(PortaViewModel model)
    {
        Log.debug("c: updating portaview model: " + model.getName());
        try
        {
            Hashtable rtn =
                (Hashtable) (getClientToHost(model
                    .getInetAddress()
                    .getHostAddress())
                    .execute("main.getStatus", new Vector()));
            SlideSettings ss = new SlideSettings(rtn);
            if (model.getSlideSettings().equals(ss))
            {
                return;
            }
            else
            {
                model.setSlideSettings(ss);
            }
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Register this PortaView with the master.
     * @return true if registration is successful, false otherwise.
     */
    public boolean registerWithMaster()
    {
        Log.debug("c: register current portaview with master");
        try
        {
            Vector p = new Vector();
            p.add(Registry.getServer().getName());
            p.add(Registry.getServer().getInetAddress().getHostAddress());
            Boolean rtn =
                (Boolean) getClientToMaster().execute("main.register", p);
            return rtn.booleanValue();
        }
        catch (Exception e)
        {
            e.printStackTrace();
            return false;
        }
    }

    /**
     * Unregister this PortaView with the master.
     * @return true if successfully unregistered from master, false otherwise.
     */
    public boolean unregisterWithMaster()
    {
        Log.debug("c: unregister current portaview with master");
        try
        {
            Vector p = new Vector();
            p.add(Registry.getServer().getName());
            getClientToMaster().execute("main.unregister", p);
            return true;
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return true;
    }

    /**
     * Retrieves the photo with a given name.
     * @param album album name
     * @param name photo name
     * @param callback async callback that handles the return of the XML-RPC call.
     */
    public void getPhotoWithName(
        String album,
        String name,
        AsyncCallback callback)
    {
        Log.debug("c: Getting photo with name " + album + ":" + name);
        try
        {
            Vector p = new Vector();
            p.add(album);
            p.add(name);
            getClientToMaster().executeAsync(
                "main.getPhotoWithName",
                p,
                callback);
        }
        catch (Exception e)
        {
            // error
            e.printStackTrace();
        }
    }

    /**
     * Returns a list of AlbumModels from the master.
     */
    public AlbumCollection getAlbumsFromMaster()
    {
        Log.debug("c: Getting albums from master.");
        Vector p = new Vector();
        try
        {
            Vector albums =
                (Vector) getClientToMaster().execute("main.getAlbums", p);
            // the return is a vector of hashes of string.  The second level 
            // of vectors are the photos associated with it.
            AlbumCollection rtn = new AlbumCollection();
            for (int i = 0; i < albums.size(); i++)
            {
                Hashtable ht = (Hashtable) albums.get(i);
                String name = (String) ht.get(Album.XT_NAME);
                AlbumModel am = new AlbumModel(name);
                Vector photos = (Vector) ht.get(Album.XT_PHOTOS);
                List rtnp = new Vector();
                for (int j = 0; j < photos.size(); j++)
                {
                    rtnp.add(new PhotoModel(am, (String) photos.get(j)));
                }
                am.setPhotos(rtnp);
                am.setImage(
                    ImageUtils.getImage((byte[]) ht.get(Album.XT_ICON)));
                rtn.add(am);
            }
            return rtn;

        }
        catch (Exception e)
        {
            // error
            e.printStackTrace();
        }
        return null;

    }

    /**
     * Updates the remote portaview with the portaview's settings.
     * @param pvm PortaViewModel that is used to update the remote PortaView.
     */
    public void updateRemotePortaView(PortaViewModel pvm)
    {
        Log.debug(
            "c: Updating remote PortaView: "
                + pvm.getName()
                + "@"
                + pvm.getInetAddress());
        Vector p = new Vector();
        p.add(pvm.getSlideSettings().toHashtable());
        try
        {
            getClientToHost(
                pvm.getInetAddress().getHostAddress()).execute(
                "main.update",
                p);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Tries to retrieve the full image from the master.  This uses an asynchronous XML-RPC call
     * to retrieve the image, since potentially it can be big and take a long time.
     */
    public void getPhotoFromMaster(SlideSettings ss, AsyncCallback callback)
    {
        Log.debug("c: Getting photo from master:" + ss);
        String slide = ss.getSlide();
        int index = ss.getIndex();
        Vector p = new Vector();
        synchronized (this)
        {
            p.add(slide);
            p.add(new Integer(index));
        }
        try
        {
            getClientToMaster().executeAsync("main.getPhoto", p, callback);
        }
        catch (Exception e)
        {
            // error
            e.printStackTrace();
        }
    }

    /**
     * Returns the client that can talk to the particular portaview host.
     * @param host the host name/IP of the remote host.
     * @return the XML-RPC client object.
     * @throws Exception when the URL generated is not valid for the given host.
     */
    private XmlRpcClient getClientToHost(String host) throws Exception
    {
        XmlRpcClient xmlrpc =
            new XmlRpcClient(
                PortaViewServer.XRPC_PROTO
                    + "://"
                    + host
                    + ":"
                    + Consts.XML_RPC_PORT
                    + "/RPC2");
        //Log.debug("Client Executing call to " + xmlrpc.getURL());
        return xmlrpc;
    }

    /**
     * Returns the XML-RPC client to the master server.
     */
    private XmlRpcClient getClientToMaster() throws Exception
    {
        if (Registry.getServer().getMasterIP() == null)
            throw new Exception("Master IP not set");
        return getClientToHost(
            Registry.getServer().getMasterIP().getHostAddress());

    }
}
