/*
 * Created on Sep 28, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.server;

import java.io.File;
import java.net.InetAddress;
import java.util.Hashtable;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import org.apache.xmlrpc.XmlRpcHandler;

import portaview.Log;
import portaview.Registry;
import portaview.model.Album;
import portaview.model.AlbumModel;
import portaview.model.Photo;
import portaview.model.PortaViewCollection;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;

/**
 * Commands for the XML-RPC server.  Note that the signatures of 
 * class must be compliant to the XML-RPC conversion table.
 * 
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: PortaViewCommands.java,v 1.9 2003/12/10 06:18:57 wlee Exp $
 */
public class PortaViewCommands implements XmlRpcHandler
{

    private PortaViewServer server = null;

    public PortaViewCommands(PortaViewServer server)
    {
        this.server = server;
    }

    /**
     * Given a show and a sequence number return the photo struct.
     * @param show show name
     * @param seq sequence number
     * @return the byte array that represents the jpeg representation of the photo.
     */
    public byte[] getPhoto(String show, int seq)
    {
        Album a = server.getRepository().getAlbum(show);
        Photo p = a.getPhoto(seq);
        return p.getJpeg();
    }

    /**
     * Given a show and a sequence number return the photo struct.
     * @param show show name
     * @param seq sequence number
     * @return the byte array that represents the jpeg representation of the photo.
     */
    public byte[] getPhotoWithName(String album, String name)
    {
        Album a = server.getRepository().getAlbum(album);
        Photo p = a.getPhoto(name);
        return p.getJpeg();
    }

    /**
     * Update this portaview main displayer.  This will also spawn a request to the 
     * master server in order to download the image.
     *
     * @param slide the hashtable contains the following key.  Here is an
     * example:
     * <pre>
     *  struct.put(ViewerConsts.TOTAL, new Integer(3)); // total number of slides
     *  struct.put(ViewerConsts.SPEED, new Integer(5)); // speed 0 = stop, >0 = second for the next slide.
     *  struct.put(ViewerConsts.BORDER_RED, new Integer(0)); // border red
     *  struct.put(ViewerConsts.BORDER_GREEN, new Integer(255)); // border green
     *  struct.put(ViewerConsts.BORDER_BLUE, new Integer(0)); // border blue
     *  struct.put(ViewerConsts.BORDER, new Integer(2)); // border thickness
     * </pre>
     * @return
     */
    public boolean update(Hashtable slide)
    {
        synchronized (this)
        {
            SlideSettings newSlide = new SlideSettings(slide);
            Registry
                .getMainApplication()
                .getThisPortaViewModel()
                .setSlideSettings(
                newSlide);
            //			server.getClient().getImageFromMaster();
            return true;
        }
    }

    /**
     * RPC call for getting the status.
     * 
     * @return A hastable like the argument for update.
     */
    public Object getStatus()
    {
        synchronized (this.server)
        {
            return (
                Registry
                    .getMainApplication()
                    .getThisPortaViewModel()
                    .getSlideSettings()
                    .toHashtable());
        }
    }

    /**
     * Returns a Vector of Hashtables that contains the album name and photo names
     * The hastable contains the "name", "icon", and "photos" key.  The name is of 
     * type String, the icon key contains a byte[], and the photos is a Vector of Strings.
     * @return a Vector of albums.
     */
    public Vector getAlbums()
    {
        Vector rtn = new Vector();
        Hashtable album = null;
        Repository rep = server.getRepository();
        List alist = rep.listAlbums();
        for (Iterator iter = alist.iterator(); iter.hasNext();)
        {
            Album a = (Album) iter.next();
            rtn.add(a.toXmlRpcType());
        }
        return rtn;
    }

    /**
     * Returns a list of albums contained in the repository.  Any folder
     * under the repository structure that contains at least 1 JPEG would be
     * considered an album.
     * 
     * @param rep The location of the repository.
     * @return a List of albums
     */
    private List listAlbums(File rep)
    {
        return null;
    }

    /**
     * Registers this PortaView with the master given the master's ip address.
     * @param masterName the Master server's name
     * @param masterIP the Master server's IP
     * @return A struct like: name=[Name of PortaView], ip=[ip address of the current portaView]
     *
     */
    public Hashtable registerMaster(String masterName, String masterIP)
        throws Exception
    {
        synchronized (this.server)
        {
            Hashtable rtn = new Hashtable();
            server.setMasterName(masterName);
            server.setMasterIP(InetAddress.getByName(masterIP));
            rtn.put("name", server.getName());
            rtn.put("ip", server.getInetAddress().getHostAddress());
            return rtn;
        }
    }

    /**
     * Only would be called if this PortaView is a master.  This is called
     * by the remote PortaView to register them to this master.
     * @param portaViewName the PortaView's name.
     * @param myIP the IP address for the PortaView.
     * @return true if successful, false otherwise.
     */
    public boolean register(String portaViewName, String myIP)
    {
        PortaViewCollection pv = server.getRegisteredPortaViews();
        synchronized (pv)
        {
            try
            {
                PortaViewModel pvm =
                    new PortaViewModel(
                        portaViewName,
                        InetAddress.getByName(myIP),
                        new AlbumModel("(No Album"));
                pv.addPortaView(pvm);
                return true;
            }
            catch (Exception e)
            {
                e.printStackTrace();
                return false;
            }
        }
    }

    /**
     * Unregister the portaview from the master's list.
     * 
     * @param portaViewName name of the PortaView.
     * @return true if successful, false otherwise.
     */
    public boolean unregister(String portaViewName)
    {
        PortaViewCollection pv = server.getRegisteredPortaViews();
        synchronized (pv)
        {
            boolean removed = false;
            for (Iterator i = pv.iterator(); i.hasNext();)
            {
                PortaViewModel pvm = (PortaViewModel) i.next();
                if (pvm.getName().equals(portaViewName))
                {
                    i.remove();
                    removed = true;
                }
            }
            if (removed)
                pv.notifyChange();
        }
        return true;
    }

    /**
     * Implements the method to dispatch the methods.  This contains the
     * lookup necessary to redirect to the right call.
     * @see org.apache.xmlrpc.XmlRpcHandler#execute(java.lang.String, java.util.Vector)
     */
    public Object execute(String method, Vector params) throws Exception
    {
        // This is a somewhat stupid handler, but it's sufficient for now.
        if (method.equals("main.registerMaster"))
        {
            Log.debug("s: Calling registerMaster with: " + params.toString());
            return registerMaster(
                (String) params.get(0),
                (String) params.get(1));
        }
        else if (method.equals("main.register"))
        {
            Log.debug("s: Calling register with: " + params.toString());
            boolean b =
                register((String) params.get(0), (String) params.get(1));
            return new Boolean(b);
        }
        else if (method.equals("main.unregister"))
        {
            Log.debug("s: Calling unregister with: " + params.toString());
            boolean b = unregister((String) params.get(0));
            return new Boolean(b);
        }
        else if (method.equals("main.getPhoto"))
        {
            Log.debug("s: Calling getPhoto with: " + params.toString());
            return getPhoto(
                (String) params.get(0),
                ((Integer) params.get(1)).intValue());
        }
        else if (method.equals("main.getPhotoWithName"))
        {
            Log.debug("s: Calling getPhotoWithName with: " + params.toString());
            return getPhotoWithName(
                (String) params.get(0),
                (String) params.get(1));
        }
        else if (method.equals("main.getStatus"))
        {
            Log.debug("s: Calling getStatus with: " + params.toString());
            return getStatus();
        }
        else if (method.equals("main.update"))
        {
            Log.debug("s: Calling update with: " + params.toString());
            boolean b = update((Hashtable) params.get(0));
            return new Boolean(b);
        }
        else if (method.equals("main.getAlbums"))
        {
            Log.debug("s: Calling getAlbums with: " + params.toString());
            return getAlbums();
        }
        return null;
    }
}
