/*
 * Created on Sep 22, 2003
 * Copyright (c) 2003.  All rights reserved.
 */
package portaview.server;

import java.net.InetAddress;

import org.apache.xmlrpc.WebServer;

import portaview.model.AlbumCollection;
import portaview.model.PortaViewCollection;

/**
 * A server that runs on the remote box that listens to the 
 * display call from the source.
 *
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: PortaViewServer.java,v 1.9 2003/12/10 06:18:57 wlee Exp $
 */
public class PortaViewServer
{

    private WebServer xmlRpcServer = null;
    private String name = null;
    private PortaViewUpdateRunner updateRunner = null;
    private SlideRunner slideRunner = null;
    public static String XRPC_PROTO = "http";

    /**
     * List of portaviews.  The slave server only contains one, but the master one 
     * should contains multiple of these.
     */
    private PortaViewCollection registeredPortaViews =
        new PortaViewCollection();
    private AlbumCollection albums = new AlbumCollection();

    /**
     * Repository, the central place that stores all the album.
     */
    private Repository repository = null;
    private String masterName = null;
    private InetAddress masterIP = null;
    private InetAddress hostIP = null;

    /**
     * Starts the XML-RPC server, the update runnder, and slide runner.
     * @throws Exception if starting fails.
     */
    public synchronized void start() throws Exception
    {
        xmlRpcServer.start();
        updateRunner.start();
        slideRunner.start();
    }

    /**
     * Shuts down all servers.
     */
    public synchronized void shutdown()
    {
        slideRunner.shutdown();
        updateRunner.shutdown();
        xmlRpcServer.shutdown();
    }

    /**
     * Constructs the an instance of the viewerserver
     */
    public PortaViewServer(
        String name,
        InetAddress hostIP,
        InetAddress masterIP,
        Repository rep,
        int port)
    {
        xmlRpcServer = new WebServer(port, hostIP);
        updateRunner = new PortaViewUpdateRunner();
        slideRunner = new SlideRunner();
        this.repository = rep;
        this.hostIP = hostIP;
        this.name = name;
        this.masterName = name;
        this.masterIP = masterIP;
        init();
    }

    /**
     * Initialize this server.
     */
    private synchronized void init()
    {
        PortaViewCommands cmd = new PortaViewCommands(this);
        xmlRpcServer.addHandler("main", cmd);
    }

    /**
     * @return the name of this server/PortaView.
     */
    public synchronized String getName()
    {
        return name;
    }

    /**
     * Sets the name of this PortaView.
     * @param name name of PortaView.
     */
    public synchronized void setName(String name)
    {
        this.name = name;
    }

    /**
     * @return the master's name.
     */
    public synchronized String getMasterName()
    {
        return masterName;
    }

    /**
     * Sets the master's name.
     * @param masterName master's name.
     */
    public synchronized void setMasterName(String masterName)
    {
        this.masterName = masterName;
    }

    /**
     * @return the master's IP address.
     */
    public synchronized InetAddress getMasterIP()
    {
        return masterIP;
    }

    /**
     * Sets the master's IP address.
     * @param masterIP
     */
    public synchronized void setMasterIP(InetAddress masterIP)
    {
        this.masterIP = masterIP;
    }

    /**
     * @return the IP address for this server.
     */
    public synchronized InetAddress getInetAddress()
    {
        return hostIP;
    }

    /**
     * Sets the IP address for this server.
     * @param inetAddress IP address.
     */
    public synchronized void setInetAddress(InetAddress inetAddress)
    {
        this.hostIP = inetAddress;
    }

    /**
     * @return the repository of photos (on disk).
     */
    public synchronized Repository getRepository()
    {
        return repository;
    }

    /**
     * Sets the repository of photos (on disk).
     * @param repository repository for photos.
     */
    public synchronized void setRepository(Repository repository)
    {
        this.repository = repository;
    }

    /**
     * @return the collection of registered PortaViews.
     */
    public PortaViewCollection getRegisteredPortaViews()
    {
        return registeredPortaViews;
    }

    /**
     * Replace the collection of registered PortaViews with a new collection.
     * @param collection the collection of PortaViews to set to.
     */
    public void setRegisteredPortaViews(PortaViewCollection collection)
    {
        // since we want to save the observer, we clear the list and add the collection.
        registeredPortaViews.clear();
        registeredPortaViews.addAll(collection);
    }

    /**
     * @return the collection of albums.
     */
    public AlbumCollection getAlbums()
    {
        return albums;
    }

    /**
     * Sets the collection of albums.
     * @param collection collection of albums.
     */
    public void setAlbums(AlbumCollection collection)
    {
        // since we want to save the observer, we clear the list and add the collection.
        albums.clear();
        albums.addAll(collection);
    }

}
