/*
 * Created on Sep 19, 2003
 * Copyright (c) 2003 William Lee.  All rights reserved.
 */
package portaview.util;

import java.awt.AlphaComposite;
import java.awt.Color;
import java.awt.Component;
import java.awt.Composite;
import java.awt.Graphics2D;
import java.awt.Image;
import java.awt.MediaTracker;
import java.awt.Toolkit;
import java.awt.geom.AffineTransform;
import java.awt.image.AffineTransformOp;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

import javax.imageio.ImageIO;

import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;

import com.sun.image.codec.jpeg.JPEGCodec;
import com.sun.image.codec.jpeg.JPEGImageDecoder;
import com.sun.image.codec.jpeg.JPEGImageEncoder;

/**
 * Image utilities to process JPG images.
 * @author <a href="mailto:wwlee1@uiuc.edu">William Lee</a>
 * @version $Id: ImageUtils.java,v 1.9 2003/12/10 06:18:56 wlee Exp $
 */
public class ImageUtils
{
    private static String[] supportedFormat = { "jpg", "png" };

    /**
     * Returns the image associated with the url
     * @param url url to get the image
     * @return the Image
     * @throws Exception when getting the image fails.
     */
    public static Image getImage(URL url) throws Exception
    {
        return Toolkit.getDefaultToolkit().getImage(url);
    }

    /**
     * Similar to getImage(URL), load from a filename instead of the URL.
     */
    public static Image getImage(String filename) throws Exception
    {
        return Toolkit.getDefaultToolkit().getImage(filename);
    }

    /**
     * Similar to getImage(String), load from a File instead of the file name.
     */
    public static Image getImage(File file) throws Exception
    {
        return Toolkit.getDefaultToolkit().getImage(file.toURL());
    }

    /**
     * Returns the image given the JPEG encoding in a byte array.
     * @param bytes byte array that contains the JPEG encoding of the image.
     * @return the Image object.
     */
    public static Image getImage(byte[] bytes)
    {
        try
        {
            return ImageIO.read(new ByteArrayInputStream(bytes));
        }
        catch (IOException e)
        {
            // error
            e.printStackTrace();
            return null;
        }
    }

    /**
     * Returns a buffered image based on the file an component to draw on.
     * @param file file to retrieve
     * @param c component to draw on
     * @return the BufferedImage object
     * @throws Exception when loading of the image fails.
     */
    public static BufferedImage getBufferedImage(File file, Component c)
        throws Exception
    {
        return getBufferedImage(getImage(file), c);
    }

    /**
     * Similar to getBufferedImage(File, Component), load the BufferedImage from an Image.
     * @param img image to load
     * @param c component to draw on
     * @return the BufferedImage object
     * @throws Exception when fails to load.
     */
    public static BufferedImage getBufferedImage(Image img, Component c)
        throws Exception
    {
        MediaTracker mt = new MediaTracker(c);
        mt.addImage(img, 0);
        mt.waitForID(0);
        BufferedImage bi =
            new BufferedImage(
                img.getWidth(null),
                img.getHeight(null),
                BufferedImage.TYPE_INT_RGB);
        Graphics2D biContext = bi.createGraphics();
        biContext.drawImage(img, 0, 0, null);
        return bi;
    }

    /**
     * Returns a buffered image, resized to fit the frame specified by fwidth and fheight.
     * Note that this will maintain the image's ratio.
     * 
     * @param bi Buffered image that contains the source for the image.
     * @param fwidth frame width
     * @param fheight height
     * @param c component to draw the image from.
     * @return A new instance of BufferedImage
     * @throws Exception
     */
    public static BufferedImage getBufferedImageInFrame(
        BufferedImage bi,
        int fwidth,
        int fheight,
        Component c)
        throws Exception
    {
        AffineTransform at = new AffineTransform();
        double scale = getScale(bi.getWidth(), bi.getHeight(), fwidth, fheight);
        at.scale(scale, scale);
        BufferedImageOp biop =
            new AffineTransformOp(at, AffineTransformOp.TYPE_BILINEAR);
        BufferedImage target =
            new BufferedImage(
                (int) (bi.getWidth() * scale),
                (int) (bi.getHeight() * scale),
                BufferedImage.TYPE_INT_RGB);
        Graphics2D writeBuffer = (Graphics2D) target.getGraphics();
        //		writeBuffer.scale(scale, scale);
        writeBuffer.drawImage(bi, biop, 0, 0);
        //writeBuffer.drawImage(bi, 0, 0, new JFrame());
        return target;
        //*/
    }

    /**
     * Returns the scale.
     * @param origWidth original width of the image
     * @param origHeight original height of the image
     * @param targetWidth target width of the image
     * @param targetHeight target height of the image
     * @return the scale from 0 to 1 for rescaling.
     */
    private static double getScale(
        int origWidth,
        int origHeight,
        int targetWidth,
        int targetHeight)
    {
        double imageRatio = (double) origWidth / (double) origHeight;
        double paintRatio = (double) targetWidth / (double) targetHeight;
        double scale = 1;
        if (paintRatio < imageRatio)
        {
            scale = (double) targetWidth / (double) origWidth;
        }
        else
        {
            scale = (double) targetHeight / (double) origHeight;
        }
        return scale;
    }

    /**
     * Returns a buffered image, resized to fit the frame specified by fwidth and fheight.
     * Note that this will maintain the image's ratio.
     * 
     * @param f file to load the image from
     * @param fwidth frame width
     * @param fheight height
     * @param c component to draw the image from.
     * @return A new instance of BufferedImage
     * @throws Exception
     */
    public static BufferedImage getBufferedImageInFrame(
        File f,
        int fwidth,
        int fheight,
        Component c)
        throws Exception
    {
        //BufferedImage bi = ImageIO.read(f);
        BufferedImage bi = getBufferedImage(f, c);
        return getBufferedImageInFrame(bi, fwidth, fheight, c);
    }

    /**
     * Returns the JPEG encoding of a scaled image.
     * @param f image file.
     * @param width desired width for the image
     * @param height desired height for image
     * @return the JPEG encoding of the scaled image.
     * @throws Exception when scaling fails.
     */
    public static byte[] getScaledImage(File f, int width, int height)
        throws Exception
    {
        InputStream in = new FileInputStream(f);
        JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(in);
        Image inImage = decoder.decodeAsBufferedImage();

        double scale =
            getScale(
                inImage.getWidth(null),
                inImage.getHeight(null),
                width,
                height);

        int scaledW = (int) (scale * inImage.getWidth(null));
        int scaledH = (int) (scale * inImage.getHeight(null));

        BufferedImage outImage =
            new BufferedImage(scaledW, scaledH, BufferedImage.TYPE_INT_RGB);

        AffineTransform tx = new AffineTransform();
        if (scale < 1.0d)
        {
            tx.scale(scale, scale);
        }

        // Paint image.
        Graphics2D g2d = outImage.createGraphics();
        g2d.drawImage(inImage, tx, null);
        g2d.dispose();

        // JPEG-encode the image and write to file.
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        JPEGImageEncoder encoder = JPEGCodec.createJPEGEncoder(out);
        encoder.encode(outImage);
        out.close();
        return out.toByteArray();
    }

    /**
     * Draws border around the graphics with the width expressed as a ratio 
     * to the whole image.
     * @param g graphic to draw on
     * @param ratio goes from 0 to 30, which means the % of the picture.
     * @param maxWidth maximum width of graphics
     * @param maxHeight maximum height of graphics
     * @param c the color of the border
     */
    public static void drawBorderRatio(
        Graphics2D g,
        float trans,
        int ratio,
        int maxWidth,
        int maxHeight,
        Color c)
    {
        drawBorderRatio(g, trans, ratio, 0, 0, maxWidth, maxHeight, c);
    /*
        int bwidth =
            (int) (((maxWidth > maxHeight ? maxWidth : maxHeight)) * ratio)
                / 100;
        drawBorder(g, trans, bwidth, maxWidth, maxHeight, c);
        */
    }

    /**
     * Draws border around the graphics with the width expressed as a ratio 
     * to the whole image.
     * @param g graphic to draw on
     * @param ratio goes from 0 to 30, which means the % of the picture.
     * @param xoffset the x coordinate offset
     * @param yoffset the y coordinate offset
     * @param maxWidth maximum width of graphics
     * @param maxHeight maximum height of graphics
     * @param c the color of the border
     */
    public static void drawBorderRatio(
        Graphics2D g,
        float trans,
        int ratio,
        int xoffset,
        int yoffset,
        int maxWidth,
        int maxHeight,
        Color c)
    {
        int bwidth =
            (int) (((maxWidth > maxHeight ? maxWidth : maxHeight)) * ratio)
                / 100;
        drawBorder(g, trans, bwidth, xoffset, yoffset, maxWidth, maxHeight, c);
    }

    /**
     * Draws border around the graphics
     * @param g graphic to draw on
     * @param bwidth border width
     * @param xoffset the x coordinate offset
     * @param yoffset the y coordinate offset
     * @param maxWidth maximum width of graphics
     * @param maxHeight maximum height of graphics
     * @param c the color of the border
     */
    public static void drawBorder(
        Graphics2D g,
        float trans,
        int bwidth,
        int xoffset,
        int yoffset,
        int maxWidth,
        int maxHeight,
        Color c)
    {
        if (bwidth == 0)
            return;
        g.setColor(c);
        Composite orig = g.getComposite();
        if (trans > 1.0F)
        {
            g.setComposite(createComposite(1.0F));
        }
        else
        {
            g.setComposite(createComposite(trans));
        }
        // from upper left to lower left
        g.fillRect(xoffset, yoffset, bwidth, maxHeight - bwidth);
        // from lower left to lower right
        g.fillRect(
            xoffset,
            yoffset + maxHeight - bwidth,
            maxWidth - bwidth,
            bwidth);
        // from upper right to lower right
        g.fillRect(
            xoffset + maxWidth - bwidth,
            yoffset + bwidth,
            bwidth,
            maxHeight);
        // from upper left to upper right
        g.fillRect(xoffset + bwidth, yoffset, maxWidth - bwidth, bwidth);
        g.setComposite(orig);
    }

    /**
     * Draws border around the graphics
     * @param g graphic to draw on
     * @param bwidth border width
     * @param maxWidth maximum width of graphics
     * @param maxHeight maximum height of graphics
     * @param c the color of the border
     */
    public static void drawBorder(
        Graphics2D g,
        float trans,
        int bwidth,
        int maxWidth,
        int maxHeight,
        Color c)
    {
        drawBorder(g, trans, bwidth, 0, 0, maxWidth, maxHeight, c);

    }

    /**
     * Create the alpha composite given a alpha value from 0 to 1.
     * @param alpha transparency.
     * @return the AlphaComposite object.
     */
    private static AlphaComposite createComposite(float alpha)
    {
        return (AlphaComposite.getInstance(AlphaComposite.SRC_OVER, alpha));
    }

    /**
     * Draw the image according to the specified settings.
     * @param pvm
     * @param bi
     */
    public static void drawBorder(PortaViewModel pvm, BufferedImage bi)
    {
        SlideSettings ss = pvm.getSlideSettings();
        Graphics2D g = (Graphics2D) bi.getGraphics();
        drawBorderRatio(
            g,
            (ss.getTransparency() / 10.0F),
            ss.getBorder(),
            bi.getWidth(),
            bi.getHeight(),
            ss.getBorderColor());
    }

}
