/*
 * Created on Oct 26, 2003
 *
 */
package portaview.view;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.Image;
import java.awt.image.BufferedImage;
import java.util.Observable;
import java.util.Observer;

import javax.swing.BorderFactory;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import portaview.AbstractViewer;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;
import portaview.util.ImageUtils;

/**
 * Abstract displayer view.  The parent view object for the displayer.
 *
 * @author William Lee
 */
public abstract class AbstractDisplayerView extends JPanel implements Observer
{
    private PortaViewModel portaViewModel = null;
    private AbstractViewer viewer = null;
    private ImageControlBar controlBar = null;
    private JLabel picLabel = null;
    private JLabel titleLabel = null;

    public AbstractDisplayerView(PortaViewModel model, AbstractViewer viewer)
    {
        this.portaViewModel = model;
        this.viewer = viewer;
        // add myself to the observer.
        model.addObserver(this);
        init();
    }

    /**
     * If the displayer has no picture, returns the text that it will display.  Children
     * classes should implement this method.
     * @return the string to display.
     */
    protected abstract String getNoPicText();

    /**
     * Initialize this view.
     */
    private void init()
    {
        if (needsLabel())
        {
            titleLabel = new JLabel();
            titleLabel.setHorizontalAlignment(SwingConstants.CENTER);
            titleLabel.setBorder(BorderFactory.createEtchedBorder());
        }

        picLabel = new JLabel(getNoPicText());
        picLabel.setHorizontalAlignment(SwingConstants.CENTER);
        picLabel.setVerticalAlignment(SwingConstants.CENTER);
        picLabel.setVerticalTextPosition(SwingConstants.BOTTOM);

        //setBackground(Color.BLACK);
        setLayout(new BorderLayout());
        controlBar = getImageControls(this.viewer);
        if (needsLabel())
        {
            add(titleLabel, BorderLayout.NORTH);
        }
        add(controlBar, BorderLayout.SOUTH);
        /*
        JPanel p = new JPanel(new GridBagLayout());
        GridBagConstraints c = new GridBagConstraints();
        c.gridheight = 1;
        c.gridwidth = 1;
        c.anchor = GridBagConstraints.CENTER;
        c.weightx = 1;
        c.weighty = 1;
        c.gridx = 0;
        c.gridy = 0;
        p.add(picLabel, c);
        */
        /*
        JPanel p = new JPanel();
        add(p, BorderLayout.CENTER);
        */
        add(picLabel, BorderLayout.CENTER);
        setPreferredSize(new Dimension(640, 480));
    }

    /**
     * Returns the image control bar.  Children classes decide what button bar it wants to 
     * put below the displayer.
     * @return image control bar.
     */
    protected abstract ImageControlBar getImageControls(AbstractViewer viewer);

    /**
     * Children should implement this in order to set whether you need the label or not.
     * @return true if children needs a label.
     */
    protected abstract boolean needsLabel();

    /**
     * @return true if the view need refreshing, false otherwise.
     */
    protected abstract boolean needRefresh();

    /**
     * Refreshes the tool bar if the model has changed.
     */
    public void refreshBar(Observable dmodel)
    {
        if (dmodel instanceof PortaViewModel)
        {
            PortaViewModel m = (PortaViewModel) dmodel;
            // depending on the model, disable the buttons.
            SlideSettings ss = m.getSlideSettings();
            if (ss.getTotal() == 0)
            {
                // disable all the buttons
                controlBar.getPrevButton().setEnabled(false);
                controlBar.getPauseButton().setEnabled(false);
                controlBar.getPlayButton().setEnabled(false);
                controlBar.getNextButton().setEnabled(false);
                JButton sb = controlBar.getSettingsButton();
                if (sb != null)
                    sb.setEnabled(false);
            }
            else
            {
                JButton sb = controlBar.getSettingsButton();
                if (ss.getPlayState() == SlideSettings.PAUSE)
                {
                    controlBar.getPrevButton().setEnabled(true);
                    controlBar.getPauseButton().setEnabled(false);
                    controlBar.getPlayButton().setEnabled(true);
                    controlBar.getNextButton().setEnabled(true);
                    if (sb != null)
                        sb.setEnabled(true);
                }
                else
                {
                    controlBar.getPrevButton().setEnabled(false);
                    controlBar.getPauseButton().setEnabled(true);
                    controlBar.getPlayButton().setEnabled(false);
                    controlBar.getNextButton().setEnabled(false);
                    if (sb != null)
                        sb.setEnabled(false);
                }
            }
        }
    }

    /**
     * Refreshes the image, resizing it to the size of the label.
     */
    public void refreshImage()
    {
        if (!needRefresh())
            return;
        if (needsLabel())
        {
            SlideSettings ss = getPortaViewModel().getSlideSettings();
            // Note that index starts from 0, we need to increment one
            if (ss.getTotal() != 0)
            {
                String txt =
                    portaViewModel.getName()
                        + " (Picture "
                        + (ss.getIndex() + 1)
                        + " of "
                        + ss.getTotal()
                        + ")";
                if (ss.getPlayState() == SlideSettings.PLAY)
                {
                    txt = txt + " (Refreshing every " + ss.getSpeed() + " sec)";
                }
                else
                {
                    txt = txt + " (Paused)";
                }
                titleLabel.setText(txt);
            }
            else
            {
                titleLabel.setText(portaViewModel.getName());
            }
        }
        Image img = this.portaViewModel.getImageModel().getImage();
        BufferedImage bi = null;
        if (img != null
            && picLabel.getWidth() > 20
            && picLabel.getHeight() > 20)
        {
            try
            {
                bi =
                    ImageUtils.getBufferedImageInFrame(
                        ImageUtils.getBufferedImage(img, picLabel),
                        picLabel.getWidth(),
                        picLabel.getHeight(),
                        picLabel);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
        if (bi != null)
        {
            ImageUtils.drawBorder(portaViewModel, bi);
            ImageIcon ii = new ImageIcon(bi);
            picLabel.setIcon(ii);
            picLabel.setText("");
            picLabel.repaint();
        }
        else
        {
            picLabel.setIcon(null);
            picLabel.setText(getNoPicText());
            picLabel.setForeground(getNoPicTextColor());
            picLabel.repaint();
        }
    }




    /**
     * Refresh the tool bar and the image.
     *
     */
    public void refresh()
    {
        refreshBar(portaViewModel);
        refreshImage();
    }

    /**
     * The model would call this method when its state has change.
     * 
     * @see java.util.Observer#update(java.util.Observable, java.lang.Object)
     */
    public void update(Observable dispModel, Object arg1)
    {
        refreshImage();
        refreshBar(dispModel);
    }

    /**
     * @return the PortaViewModel associated with this view.
     */
    public PortaViewModel getPortaViewModel()
    {
        return portaViewModel;
    }

    /**
     * Sets the PortaViewModel
     * @param model the model to set.
     */
    public void setPortaViewModel(PortaViewModel model)
    {
        // unregister the listener
        if (portaViewModel != null)
            portaViewModel.deleteObserver(this);
        portaViewModel = model;
        model.addObserver(this);
        refresh();
    }

    protected Color getNoPicTextColor()
    {
        return Color.black;
    }

}
