/*
 * Created on Oct 27, 2003
 */
package portaview.view;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.dnd.DropTarget;
import java.awt.dnd.DropTargetDragEvent;
import java.awt.dnd.DropTargetDropEvent;
import java.awt.dnd.DropTargetEvent;
import java.awt.dnd.DropTargetListener;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.net.URL;

import javax.swing.TransferHandler;

import portaview.Log;
import portaview.Registry;
import portaview.dnd.AlbumTransferHandler;
import portaview.model.PortaViewModel;
import portaview.model.SlideSettings;
import portaview.util.ImageUtils;

/**
 * Represents a PortaView icon listed on the left.
 * 
 * @author William Lee
 */
public class PortaViewView extends IconView implements DropTargetListener
{
    private PortaViewModel portaViewModel = null;
    public static Image emptyImage = null;
    private static final int WIDTH = 150;
    private static final int HEIGHT = 150;

    public PortaViewView(PortaViewModel pview, PortaViewPanel pvp)
    {
        super(pvp);
        this.portaViewModel = pview;
        this.portaViewModel.addObserver(this);
        installDnD();
        installListener();
        initEmptyImage();
        setPreferredSize(new Dimension(WIDTH, HEIGHT));
    }

    /**
     * Sets the tool tips information
     */
    protected void doRefresh()
    {
        SlideSettings ss = portaViewModel.getSlideSettings();
        if (ss.getTotal() == 0)
            setToolTipText("No album is showing here.");
        else
        {
            String tipText =
                portaViewModel.getName()
                    + " (Album: "
                    + ss.getSlide()
                    + " "
                    + (ss.getIndex() + 1)
                    + " of "
                    + ss.getTotal()
                    + ")";
            if (ss.getPlayState() == SlideSettings.PAUSE)
            {
                tipText = tipText + " [Paused]";
            }
            else
            {
                tipText =
                    tipText
                        + " [Playing, refreshing every "
                        + ss.getSpeed()
                        + " seconds ]";
            }
            setToolTipText(tipText);
        }
    }

    /**
     * Installs the listener for the button click event.
     *
     */
    protected void installListener()
    {
        // if the button is clicked, then we want to change the Previewer
        // to change its controller.
        addActionListener(new ActionListener()
        {

            public void actionPerformed(ActionEvent e)
            {
                PortaViewView pvv = (PortaViewView) e.getSource();
                Registry.getMainApplication().getPreview().setModel(
                    pvv.getPortaViewModel());
            }
        });
    }

    protected void initEmptyImage()
    {
        if (emptyImage == null)
        {
            URL url =
                PortaViewView.class.getResource("/portaview/images/empty.jpg");
            try
            {
                Log.debug("Loading empty image: " + url);
                Image fullSize = ImageUtils.getImage(url);
                emptyImage =
                    ImageUtils.getBufferedImageInFrame(
                        ImageUtils.getBufferedImage(fullSize, this),
                        DEFAULT_WIDTH,
                        DEFAULT_HEIGHT,
                        this);
            }
            catch (Exception e)
            {
                e.printStackTrace();
            }
        }
    }

    /**
     * Adds the drag and drop support for the icon view.
     *
     */
    protected void installDnD()
    {
        setDropTarget(new DropTarget(this, this));
        setTransferHandler(new AlbumTransferHandler());
    }

    /**
     * @return the PortaView model
     */
    public PortaViewModel getPortaViewModel()
    {
        return portaViewModel;
    }

    /**
     * Sets the model.
     * @param model model to set.
     */
    public void setPortaViewModel(PortaViewModel model)
    {
        portaViewModel = model;
    }

    /**
     * Returns the icon image.
     * @see portaview.view.IconView#getIconImage()
     */
    protected Image getIconImage()
    {
        Image img = portaViewModel.getImageModel().getImage();
        if (img == null)
            return emptyImage;
        try
        {
            return ImageUtils.getBufferedImageInFrame(
                ImageUtils.getBufferedImage(img, this),
                DEFAULT_WIDTH,
                DEFAULT_HEIGHT,
                this);
        }
        catch (Exception e)
        {
            e.printStackTrace();
        }
        return emptyImage;
    }
    /**
     * Returns the icon label.
     * @see portaview.view.IconView#getIconLabel()
     */
    protected String getIconLabel()
    {
        String name = null;
        /*
        if (portaViewModel
            .equals(Registry.getMainApplication().getThisPortaViewModel()))
            name = "( " + portaViewModel.getName() + " )";
        else
            name = portaViewModel.getName();
            */
        name = portaViewModel.getName();
        String aname = null;
        SlideSettings ss = portaViewModel.getSlideSettings();
        if (ss.getSlide().equals(""))
        {
            aname = "No Album";
            String[] lbls = { name, aname };
            return getMultiLineLabel(lbls);
        }
        else
        {
            aname = "" + ss.getSlide();
        }
        String pic = " [" + (ss.getIndex() + 1) + "/" + ss.getTotal() + "]";
        aname = aname + pic;
        String state = null;
        if (ss.getPlayState() == SlideSettings.PAUSE)
        {
            state = "- Paused -";
        }
        else
        {
            state = "- Playing -";
        }
        String[] lbls = { name, aname, state };
        return getMultiLineLabel(lbls);
    }

    /**
     * Does nothing.
     * @see java.awt.dnd.DropTargetListener#dragEnter(java.awt.dnd.DropTargetDragEvent)
     */
    public void dragEnter(DropTargetDragEvent dtde)
    {
    }

    /**
     * Does nothing
     * @see java.awt.dnd.DropTargetListener#dragOver(java.awt.dnd.DropTargetDragEvent)
     */
    public void dragOver(DropTargetDragEvent dtde)
    {
    }

    /**
     * Does nothing
     * @see java.awt.dnd.DropTargetListener#dropActionChanged(java.awt.dnd.DropTargetDragEvent)
     */
    public void dropActionChanged(DropTargetDragEvent dtde)
    {

    }

    /**
     * Perform the drop by setting the slide settings.
     * @see java.awt.dnd.DropTargetListener#drop(java.awt.dnd.DropTargetDropEvent)
     */
    public void drop(DropTargetDropEvent dtde)
    {
        TransferHandler th = this.getTransferHandler();
        th.importData(this, dtde.getTransferable());
    }

    /**
     * Does nothing
     * @see java.awt.dnd.DropTargetListener#dragExit(java.awt.dnd.DropTargetEvent)
     */
    public void dragExit(DropTargetEvent dte)
    {
    }

    /**
     * Returns true only if the display is not active.
     * @see portaview.view.IconView#needRefresh()
     */
    protected boolean needRefresh()
    {
        return (!Registry.getMainApplication().isDisplayActive());
    }

    /**
     * Process the image in order to add a border to the icon.
     * 
     * @author William Lee
     */
    protected Image postProcessIcon(Image img)
    {
        ImageUtils.drawBorder(portaViewModel, (BufferedImage) img);
        return img;
    }

}
